package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.biganto.visual.roompark.data.local.UserState
import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.data.repository.mapper.fromRawList
import com.biganto.visual.roompark.domain.contract.DealContract
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.domain.model.EstateModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */
//


class EstateRepository @Inject constructor(
    private val local: ILocalStore,
    private val api: IRoomParkApi,
    private val db: IDb
): DealContract {
    init {
        Timber.d("Estate Repository Created")
    }

    private val getFavoritesApi: Observable<List<EstateEntity>> =
        local.recentUser()
            .flatMap {
                when (it) {
                    is UserState.Authenticated -> db.fetchUser(it.uuid.toInt())
                    else -> throw CustomApiException.NotAuthorizedException()
                }
            }
            .flatMap { user ->
                api.getFavorites(user.authToken)
                    .doOnError(Timber::e)
                    .map { fromRawList(it, ::fromRaw) }
                    .doOnNext {
                        it.forEach { estate ->
                            estate.setFavorite(true)
                            estate.user = user
                        }
                    }
                    .doOnNext(db::blockingUpsert)
            }

    private val getFavoritesDb: Observable<List<EstateEntity>> =
        local.recentUser()
            .flatMap {
                when (it) {
                    is UserState.Authenticated -> db.fetchUser(it.uuid.toInt()).take(1)
                    else -> throw CustomApiException.NotAuthorizedException()
                }
            }
            .flatMap {
                db.getUserFavorites(it.uuid)
                    .doOnError(Timber::e)
                    .toList().toObservable()
            }


    override fun getFavorites(): Observable<List<EstateModel>> {
        return Observable.mergeDelayError(
            arrayListOf(
                getFavoritesApi, getFavoritesDb
            )
        ).map { fromEntity(it, ::fromEntity) }
            .doOnError(Timber::e)
    }


}
